Type: Package
Package: openxlsx
Title: Read, Write and Edit xlsx Files
Version: 4.2.8.1
Date: 2025-10-30
Authors@R: 
    c(person(given = "Philipp",
             family = "Schauberger",
             role = "aut",
             email = "philipp@schauberger.co.at"),
      person(given = "Alexander",
             family = "Walker",
             role = "aut",
             email = "Alexander.Walker1989@gmail.com"),
      person(given = "Luca",
             family = "Braglia",
             role = "ctb"),
      person(given = "Joshua",
             family = "Sturm",
             role = "ctb"),
      person(given = "Jan Marvin",
             family = "Garbuszus",
             role = c("ctb", "cre"),
             email = "jan.garbuszus@ruhr-uni-bochum.de"),
      person(given = "Jordan Mark",
             family = "Barbone",
             role = "ctb",
             email = "jmbarbone@gmail.com",
             comment = c(ORCID = "0000-0001-9788-3628")),
      person(given = "David",
             family = "Zimmermann",
             role = "ctb",
             email = "david_j_zimmermann@hotmail.com"),
      person(given = "Reinhold",
             family = "Kainhofer",
             role = "ctb",
             email = "reinhold@kainhofer.com"))
Description: Simplifies the creation of Excel .xlsx files by providing a
    high level interface to writing, styling and editing worksheets.
    Through the use of 'Rcpp', read/write times are comparable to the
    'xlsx' and 'XLConnect' packages with the added benefit of removing the
    dependency on Java.
License: MIT + file LICENSE
URL: https://ycphs.github.io/openxlsx/index.html,
        https://github.com/ycphs/openxlsx
BugReports: https://github.com/ycphs/openxlsx/issues
Depends: R (>= 3.3.0)
Imports: grDevices, methods, Rcpp, stats, stringi, utils, zip
Suggests: curl, formula.tools, knitr, rmarkdown, testthat
LinkingTo: Rcpp
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
Collate: 'CommentClass.R' 'HyperlinkClass.R' 'RcppExports.R'
        'class_definitions.R' 'StyleClass.R' 'WorkbookClass.R'
        'asserts.R' 'baseXML.R' 'borderFunctions.R' 'build_workbook.R'
        'chartsheet_class.R' 'conditional_formatting.R'
        'data-fontSizeLookupTables.R' 'helperFunctions.R'
        'loadWorkbook.R' 'onUnload.R' 'openXL.R' 'openxlsx-package.R'
        'openxlsx.R' 'openxlsxCoerce.R' 'readWorkbook.R'
        'setWindowSize.R' 'sheet_data_class.R' 'utils.R'
        'workbook_column_widths.R' 'workbook_read_workbook.R'
        'workbook_write_data.R' 'worksheet_class.R' 'wrappers.R'
        'writeData.R' 'writeDataTable.R' 'writexlsx.R' 'zzz.R'
LazyData: true
NeedsCompilation: yes
Packaged: 2025-10-30 20:28:52 UTC; janmarvingarbuszus
Author: Philipp Schauberger [aut],
  Alexander Walker [aut],
  Luca Braglia [ctb],
  Joshua Sturm [ctb],
  Jan Marvin Garbuszus [ctb, cre],
  Jordan Mark Barbone [ctb] (ORCID:
    <https://orcid.org/0000-0001-9788-3628>),
  David Zimmermann [ctb],
  Reinhold Kainhofer [ctb]
Maintainer: Jan Marvin Garbuszus <jan.garbuszus@ruhr-uni-bochum.de>
Repository: CRAN
Date/Publication: 2025-10-31 06:10:51 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 09:33:17 UTC; unix
Archs: openxlsx.so.dSYM
