# Generated by roxygen2: do not edit by hand

S3method("names<-",Workbook)
S3method(as.character,formula)
S3method(getNamedRegions,Workbook)
S3method(getNamedRegions,default)
S3method(names,Workbook)
S3method(read.xlsx,Workbook)
S3method(read.xlsx,default)
export("activeSheet<-")
export("sheetVisibility<-")
export("sheetVisible<-")
export("worksheetOrder<-")
export(activeSheet)
export(addCreator)
export(addFilter)
export(addStyle)
export(addWorksheet)
export(auto_heights)
export(buildWorkbook)
export(cloneWorksheet)
export(col2int)
export(conditionalFormat)
export(conditionalFormatting)
export(convertFromExcelRef)
export(convertToDate)
export(convertToDateTime)
export(copyWorkbook)
export(createComment)
export(createNamedRegion)
export(createStyle)
export(createWorkbook)
export(dataValidation)
export(deleteData)
export(deleteDataColumn)
export(deleteNamedRegion)
export(freezePane)
export(getBaseFont)
export(getCellRefs)
export(getCreators)
export(getDateOrigin)
export(getNamedRegions)
export(getSheetNames)
export(getStyles)
export(getTables)
export(getWindowSize)
export(get_worksheet_entries)
export(groupColumns)
export(groupRows)
export(insertImage)
export(insertPlot)
export(int2col)
export(loadWorkbook)
export(makeHyperlinkString)
export(mergeCells)
export(modifyBaseFont)
export(op.openxlsx)
export(openXL)
export(openxlsx_getOp)
export(openxlsx_setOp)
export(pageBreak)
export(pageSetup)
export(protectWorkbook)
export(protectWorksheet)
export(read.xlsx)
export(readWorkbook)
export(removeCellMerge)
export(removeColWidths)
export(removeComment)
export(removeFilter)
export(removeRowHeights)
export(removeTable)
export(removeWorksheet)
export(renameWorksheet)
export(replaceStyle)
export(saveWorkbook)
export(setColWidths)
export(setFooter)
export(setHeader)
export(setHeaderFooter)
export(setLastModifiedBy)
export(setRowHeights)
export(setWindowSize)
export(sheetVisibility)
export(sheetVisible)
export(sheets)
export(showGridLines)
export(temp_xlsx)
export(ungroupColumns)
export(ungroupRows)
export(worksheetOrder)
export(write.xlsx)
export(writeComment)
export(writeData)
export(writeDataTable)
export(writeFormula)
import(Rcpp)
import(methods)
import(stringi)
importFrom(grDevices,bmp)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colours)
importFrom(grDevices,dev.copy)
importFrom(grDevices,dev.list)
importFrom(grDevices,dev.off)
importFrom(grDevices,jpeg)
importFrom(grDevices,png)
importFrom(grDevices,rgb)
importFrom(grDevices,tiff)
importFrom(stats,pchisq)
importFrom(utils,download.file)
importFrom(utils,head)
importFrom(utils,menu)
importFrom(utils,unzip)
importFrom(zip,zipr)
useDynLib(openxlsx, .registration=TRUE)
