# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

calc_column_widths <- function(sheet_data, sharedStrings, autoColumns, widths, baseFontCharWidth, minW, maxW) {
    .Call(`_openxlsx_calc_column_widths`, sheet_data, sharedStrings, autoColumns, widths, baseFontCharWidth, minW, maxW)
}

convert_to_excel_ref <- function(cols, LETTERS) {
    .Call(`_openxlsx_convert_to_excel_ref`, cols, LETTERS)
}

convert_from_excel_ref <- function(x) {
    .Call(`_openxlsx_convert_from_excel_ref`, x)
}

convert_to_excel_ref_expand <- function(cols, LETTERS, rows) {
    .Call(`_openxlsx_convert_to_excel_ref_expand`, cols, LETTERS, rows)
}

isInternalHyperlink <- function(x) {
    .Call(`_openxlsx_isInternalHyperlink`, x)
}

write_file <- function(head = "", body = "", tail = "", fl = "") {
    .Call(`_openxlsx_write_file`, head, body, tail, fl)
}

cppReadFile <- function(xmlFile) {
    .Call(`_openxlsx_cppReadFile`, xmlFile)
}

read_file_newline <- function(xmlFile) {
    .Call(`_openxlsx_read_file_newline`, xmlFile)
}

get_letters <- function() {
    .Call(`_openxlsx_get_letters`)
}

markUTF8 <- function(x, clone) {
    .Call(`_openxlsx_markUTF8`, x, clone)
}

loadworksheets <- function(wb, styleObjects, xmlFiles, is_chart_sheet) {
    .Call(`_openxlsx_loadworksheets`, wb, styleObjects, xmlFiles, is_chart_sheet)
}

getNodes <- function(xml, tagIn) {
    .Call(`_openxlsx_getNodes`, xml, tagIn)
}

getOpenClosedNode <- function(xml, open_tag, close_tag) {
    .Call(`_openxlsx_getOpenClosedNode`, xml, open_tag, close_tag)
}

getAttr <- function(x, tag) {
    .Call(`_openxlsx_getAttr`, x, tag)
}

getChildlessNode_ss <- function(xml, tag) {
    .Call(`_openxlsx_getChildlessNode_ss`, xml, tag)
}

getChildlessNode <- function(xml, tag) {
    .Call(`_openxlsx_getChildlessNode`, xml, tag)
}

get_extLst_Major <- function(xml) {
    .Call(`_openxlsx_get_extLst_Major`, xml)
}

cell_ref_to_col <- function(x) {
    .Call(`_openxlsx_cell_ref_to_col`, x)
}

int_2_cell_ref <- function(cols) {
    .Call(`_openxlsx_int_2_cell_ref`, cols)
}

get_shared_strings <- function(xmlFile, isFile) {
    .Call(`_openxlsx_get_shared_strings`, xmlFile, isFile)
}

getCellInfo <- function(xmlFile, sharedStrings, skipEmptyRows, startRow, rows, getDates) {
    .Call(`_openxlsx_getCellInfo`, xmlFile, sharedStrings, skipEmptyRows, startRow, rows, getDates)
}

read_workbook <- function(cols_in, rows_in, v, string_inds, is_date, hasColNames, hasSepNames, skipEmptyRows, skipEmptyCols, nRows, clean_names) {
    .Call(`_openxlsx_read_workbook`, cols_in, rows_in, v, string_inds, is_date, hasColNames, hasSepNames, skipEmptyRows, skipEmptyCols, nRows, clean_names)
}

calc_number_rows <- function(x, skipEmptyRows) {
    .Call(`_openxlsx_calc_number_rows`, x, skipEmptyRows)
}

map_cell_types_to_integer <- function(t) {
    .Call(`_openxlsx_map_cell_types_to_integer`, t)
}

map_cell_types_to_char <- function(t) {
    .Call(`_openxlsx_map_cell_types_to_char`, t)
}

build_cell_types_integer <- function(classes, n_rows) {
    .Call(`_openxlsx_build_cell_types_integer`, classes, n_rows)
}

buildCellTypes <- function(classes, nRows) {
    .Call(`_openxlsx_buildCellTypes`, classes, nRows)
}

build_cell_merges <- function(comps) {
    .Call(`_openxlsx_build_cell_merges`, comps)
}

buildCellList <- function(r, t, v) {
    .Call(`_openxlsx_buildCellList`, r, t, v)
}

#' @import Rcpp
write_worksheet_xml <- function(prior, post, sheet_data, R_fileName) {
    .Call(`_openxlsx_write_worksheet_xml`, prior, post, sheet_data, R_fileName)
}

buildMatrixNumeric <- function(v, rowInd, colInd, colNames, nRows, nCols) {
    .Call(`_openxlsx_buildMatrixNumeric`, v, rowInd, colInd, colNames, nRows, nCols)
}

buildMatrixMixed <- function(v, rowInd, colInd, colNames, nRows, nCols, charCols, dateCols) {
    .Call(`_openxlsx_buildMatrixMixed`, v, rowInd, colInd, colNames, nRows, nCols, charCols, dateCols)
}

matrixRowInds <- function(indices) {
    .Call(`_openxlsx_matrixRowInds`, indices)
}

build_table_xml <- function(table, tableStyleXML, ref, colNames, showColNames, withFilter) {
    .Call(`_openxlsx_build_table_xml`, table, tableStyleXML, ref, colNames, showColNames, withFilter)
}

write_worksheet_xml_2 <- function(prior, post, sheet_data, row_heights_ = NULL, outline_levels_ = NULL, R_fileName = "output") {
    .Call(`_openxlsx_write_worksheet_xml_2`, prior, post, sheet_data, row_heights_, outline_levels_, R_fileName)
}

