#' Font Size Lookup tables
#'
#' Lookup tables for font size
#'
#' @format A data.frame with column names corresponding to font names
#' @examples
#' data(openxlsxFontSizeLookupTable)
#' data(openxlsxFontSizeLookupTableBold)
"openxlsxFontSizeLookupTable"

#' @rdname openxlsxFontSizeLookupTable
#' @format NULL
"openxlsxFontSizeLookupTableBold"
