% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers.R
\name{setLastModifiedBy}
\alias{setLastModifiedBy}
\title{Set the author who modified the file last.}
\usage{
setLastModifiedBy(wb, LastModifiedBy)
}
\arguments{
\item{wb}{A workbook object}

\item{LastModifiedBy}{A string object with the name of the LastModifiedBy-User}
}
\description{
Just a wrapper of wb$changeLastModifiedBy()
}
\examples{

wb <- createWorkbook()
setLastModifiedBy(wb, "test")
}
\author{
Philipp Schauberger
}
