% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers.R
\name{getCreators}
\alias{getCreators}
\title{Get the names of the authors from the meta data of the file.}
\usage{
getCreators(wb)
}
\arguments{
\item{wb}{A workbook object}
}
\value{
vector of creators
}
\description{
Just a wrapper of wb$getCreators()
}
\examples{

wb <- createWorkbook()
getCreators(wb)
}
\author{
Philipp Schauberger
}
